// RuleSet.h : Declaration of the CRuleSet

//

// 11/19/03 Rich Williams - modified for use in SEEK

//



#ifndef __RULESET_H_

#define __RULESET_H_



#include "resource.h"       // main symbols



#include "Utilities.h"

#include "Rule.h"



#include <scew.h>



class EnvCell;

class EnvCellSet;

class EnvLayerSet;

class EnvLayer;

class Rule;



// ========================================================================

// RuleSet

// ========================================================================

class RuleSet

{

  // friend classes

  friend class Algorithm;

  friend class GarpAlgorithm;

  friend class CAlgorithm;

  friend class CRuleSet;



private:

  Rule * objRules[MAX_RULES];

  int intRules;



  // layer used to generate the model

  EnvLayerSet * objModelLayerSet;



  // temp data structures to hold verify results

  int iConfMatrix[2][2];

  int iTotalPoints;



  // counter for the number of cells

  int iTotalArea;			// total cells not masked

  int iPresenceArea;      // total cells predicted present

  int iAbsenceArea;       // total cells predicted absent

  int iNonPredictedArea;  // total cells non-predicted



  void resetConfMatrix(EnvCellSet * objTestDataset);

  void addConfMatrix(int iPredictedValue, int iActualValue);



public:

  RuleSet();

  virtual ~RuleSet();



  void loadText(char * strFilename);

  void saveText(char * strFilename);



  void fromXML(scew_element const * pXMLSetNode);

  void fromXML(char * strXML);

  char * toXML(char * id);

  void toXMLFile(char * strOutputFilename, char *id);

  void fromXMLFile(char * strInputFilename);





  // verify related methods

  void verify(EnvCellSet * objTestDataset, double dAccLimit);

  int getConfMatrix(int iPredictedValue, int iActualValue);

  int getTotalPoints();

  int getPredictedPoints();

  int getUnpredictedPoints();

  double getAccuracy();

  double getOverallAccuracy();



  void setActiveGenes(bool * bGeneIsActivePtr, int * iGeneIndexPtr, int iActiveGenesAux);



  char * translate(double MinUsage);

  void predict(EnvLayer * objPredictionLayer, EnvLayerSet * objPredictionArea, double Accuracylimit);

  void predictRuleCoverage(EnvLayer * objPredictionLayer, EnvLayerSet * objPredictionArea, double Accuracylimit);

  int applyRulesToCell(EnvCell * cell, double Accuracylimit);



  bool projectRuleSet(EnvLayerSet * oToLayerSet, bool bConservative, RuleSet * oToRuleSet);



  void updateRuleUsage();



  void setModelLayerSet(EnvLayerSet * objLyrSet);

  EnvLayerSet * getModelLayerSet();



  int size();

  void add(Rule * objRule);

  void set(int index, Rule * objRule);

  Rule * get(int index);



  void clear();

  void sort(int intPerfIndex);

  void trim(int intMaxRules);

  void discardRules(int iPerfIndex, double dValue);



  void setEvaluation(bool value);

  void setPad(char pad);

  int countPad(char pad);



  double getOveralPerformance(int iPerfIndex, int iFirstRulesToBeIncluded);

};



// ========================================================================



#endif //__RULESET_H_

