/*
 *
 *  Copyright (C) 2001 Numeric Solutions.  All Rights Reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  Further, this software is distributed without any warranty that it is
 *  free of the rightful claim of any third person regarding infringement
 *  or the like.  Any license provided herein, whether implied or
 *  otherwise, applies only to this software file.  Patent licenses, if
 *  any, provided herein do not apply to combinations of this program with
 *  other software, or any other product whatsoever.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact information: Numeric Solutions support@numericsolutions.com, or:
 *
 *  http://www.numericsolutions.com
 *
 */

#include "iostream.h"

#include <stdio.h>
#include <math.h>
#include <assert.h>

#include <fstream>
#include <string>
#include <vector>

#include <nsgeodata.h>
#include <nsgeomath.h>
#include <nsgrid.h>

#include "GeomodelGlue.h"

#include <jni.h>

using namespace std;

extern std::vector <string> getStringVector(JNIEnv *env, jobjectArray array);
extern jobjectArray getJObjectArray(JNIEnv *env, vector<string> in);
/*
 * Class:     com_numericsolutions_geomodeltools_GeomodelGlue
 * Method:    gridDensityByArrayDimension
 * Signature: ([Ljava/lang/String;FFFFFFFF)[Ljava/lang/String;
 */
JNIEXPORT jobjectArray JNICALL Java_com_numericsolutions_geomodeltools_GeomodelGlue_gridDensityByArrayDimension(JNIEnv *env,
                jclass clz, jobjectArray data, jfloat rad, jfloat xmin, jfloat xmax, jfloat ymin, jfloat ymax,
                jfloat dx, jfloat dy, jfloat p) {
  cout<<"in Java_com_numericsolutions_geomodeltools_GeomodelGlue_gridDensityByArrayDimension"<<endl;
  vector<string> dv = getStringVector(env, data);
  vector<string> gv = getNS2DGrid(xmin, xmax, ymin, ymax, dx, dy, p);
  vector<string> out = gridDensity(gv, dv, rad);
  return getJObjectArray(env, out);
}

/*
 * Class:     com_numericsolutions_geomodeltools_GeomodelGlue
 * Method:    gridDensityByArray
 * Signature: ([Ljava/lang/String;[Ljava/lang/String;F)V
 */
JNIEXPORT jobjectArray JNICALL Java_com_numericsolutions_geomodeltools_GeomodelGlue_gridDensityByArray(JNIEnv *env, jclass clz, jobjectArray grid, jobjectArray data, jfloat radius) {
  cout<<"in Java_com_numericsolutions_geomodeltools_GeomodelGlue_gridDensityByArray"<<endl;
  vector<string> gv = getStringVector(env, grid);
  vector<string> dv = getStringVector(env, data);
  // do the gridding
  vector<string> out = gridDensity(gv, dv, radius);
  return getJObjectArray(env, out);
}


/*
 * Class:     com_numericsolutions_geomodeltools_GeomodelGlue
 * Method:    convertNS2Grid_ESRIArc
 * Signature: ([Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_numericsolutions_geomodeltools_GeomodelGlue_convertNS2Grid_1ESRIArc
  (JNIEnv *env, jclass clz, jobjectArray grid, jstring filename)
{
	cout<<"in Java_com_numericsolutions_geomodeltools_GeomodelGlue_convertNS2Grid_1ESRIArc"<<endl;
	jboolean isCopy;
	vector<string> gv = getStringVector(env, grid);

	const char * s = env->GetStringUTFChars(filename, &isCopy);
	// call function in nsgrid.cpp that takes a vector containing the grid and
	// string representation of the filename
	convertNS2Grid_ESRIArc(gv, s);
}


/////////// private functions

/**
 * converts a c++ vector of strings into a java String object array
 */
 jobjectArray getJObjectArray(JNIEnv *env, vector<string> in) {
   jclass clz;
   jstring jstr;
   clz = env->FindClass("java/lang/String");
   jobjectArray newArr;
   int size = in.size();
   newArr = env->NewObjectArray(size, clz, NULL);
   for (int ii=0; ii<in.size(); ii++)
  {
    char const * ssLine;
    //cout<<"in getJObjectArray "<<ssLine<<endl;
    //convert to the C lib for use of
    //string manipulations
    ssLine = in[ii].c_str();
    //char cs[256];
    //sscanf(ssLine,  "%s",  cs);
    jstr = env->NewStringUTF(ssLine);
    env->SetObjectArrayElement(newArr, ii, jstr);
    env->DeleteLocalRef(jstr);
  }
   return newArr;
 }


/**
 * takes a java array of strings and returns a c++ vector of strings
 * @param env -- the jvm env
 * @param array -- the java array of strings
 * @return vec -- a c++ vector of strings
 */
vector<string> getStringVector(JNIEnv *env, jobjectArray array) {
  vector<string> out;
  //jclass string_class = env->FindClass("String");
  jsize len = env->GetArrayLength(array);
  int i, sum = 0;
  //jint *body = env->GetIntArrayElements(grid, 0);
  for (i=0; i<len; i++) {
    jstring myObject = (jstring)env->GetObjectArrayElement(array, i);
    /* Convert the object just obtained into a String */
    const char *str = env->GetStringUTFChars(myObject,0);
    out.push_back(str);
    //cout<< str <<endl;
    env->ReleaseStringUTFChars(myObject, str);
  }
  return out;
}


