#include <stdio.h>
#include <string>
#include <fstream>
#include <vector>
#include <math.h>
#include <assert.h>
#include <iostream>
#include <ctype.h>

#include "nsgrid.h" 

using namespace std;

char * util_ftochar(float val);
char * util_itochar(int i);
extern std::vector<string> getNS2DGrid( float xmin, float xmax, float ymin, float ymax, float dx, float dy, float p);
extern void convertNS2Grid_ESRIArc(vector<string> ns2grid, string filename);
extern float getNS2GridXRows(vector<string> ns2grid);
extern float getNS2GridYCols(vector<string> ns2grid);
extern float util_getVectorMin(vector<string> invec);
extern float util_getVectorMax(vector<string> invec);

extern float getNS2GridXMin(vector<string> ns2grid);
extern float getNS2GridYMin(vector<string> ns2grid);

extern float getNS2GridDX(vector<string> ns2grid);


/**
 * function that takes in two vectors 
 */
 vector<string> reduceStringVec(vector<int> levelsToRemove, vector<string> removeFrom){
 	//cerr<<"\x1b[2J"<<"@@@@@@@@@@@@@@@@@ start @@@@@@@@@@@@@@@@@@@@@"<<endl;
 	vector<string> newvec;
 	for(int i=1;i<=removeFrom.size();i++){
 		for(int ii=1;ii<=levelsToRemove.size();ii++){
 			int level = levelsToRemove[ii];
 			if ( i != level ) {
 			  newvec.push_back(removeFrom[i]);	
 			}
 		}
 	}
 	return newvec;
 }

/******************************************************************************
 * function that transforms an ns grid (like the output of the function:
 * getNS2DGrid) into and ESRI ascii grid.  This function is basically a rewrite
 * of the java application called:  ArcGridFile.java
 * Parameters:
 * ns2grid -- a vector that contains the data and is consistent with the output
 * of the getNS2DGrid and looks like:
 * x, y, z, xrow, ycol
 * 1214900.000000 620700.000000 300 1 1
 * 1215300.000000 620700.000000 300 2 1
 * 1215700.000000 620700.000000 300 3 1
 * 1216100.000000 620700.000000 300 4 1
 * 1216500.000000 620700.000000 300 5 1
 * 1216900.000000 620700.000000 300 6 1
 * 1217300.000000 620700.000000 300 7 1
 * 1217700.000000 620700.000000 300 8 1
 * ***************************************************************************/
void convertNS2Grid_ESRIArc(vector<string> ns2grid, string filename)
{
	static ofstream out_file;
	
  char x[20];
  char y[20];
  char z[20];
  char row[20];
  char col[20];

  vector<string> z_vec;
  vector<int> row_vec;
  vector<int> col_vec;

	// poopulate the vectors that are to be used for searching
  for(int k=0; k<ns2grid.size(); k++) {
	  char const * sLine;
    sLine = ns2grid[k].c_str();
		sscanf(sLine,  "%s %s %s %s %s",  x, y, z, row, col);
    z_vec.push_back(z);
    row_vec.push_back(atoi(row));
    col_vec.push_back(atoi(col));
  }
  
  float rows = getNS2GridXRows(ns2grid);
  float cols = getNS2GridYCols(ns2grid);
  float minx = getNS2GridXMin(ns2grid);
  float miny = getNS2GridYMin(ns2grid);
  float cellSize = getNS2GridDX(ns2grid);
 
  out_file.open((const char*)filename.c_str(), ios::out); //open the file
   
  char longLine[200000]; //this is the longline in the esri arc file
  vector<string> matrix; // this is the matrix of data stored in the right order for output to the esri grid

	int last = 0;
	for(int i=1;i<=cols;i++)
	{
		int current = (int)((i/cols)*100);
		if ( last != current ) {
			cerr<<"\x1b[2J"<<(int)((i/cols)*100)<<"% done"<<endl;
			last = (int)((i/cols)*100);
		}
		for(int j=1;j<=rows;j++)
		{
         for(int k=0; k<z_vec.size(); k++)
         {
						// if this is the node that we're searching for
            if ( i==col_vec[k] && j==row_vec[k] ) { 
              char const * sLine;
              sLine = z_vec[k].c_str();
              sscanf(sLine, "%s", z);
              //if the first row start another element in vec
              if (j==1) {
                strcpy(longLine, z);
              } else {
                strcat(longLine, " ");
                strcat(longLine, z);
              }
              break;
            }
         }
    }
    matrix.push_back(longLine);
  }
  //// header 
 	cout<<"ncols  "<<rows<<endl;
  out_file.write("ncols ", 6);
  out_file.write(util_itochar((int)(rows)), strlen(util_itochar((int)rows)));
  out_file.write(" \n", 2);
  
  cout<<"nrows  "<<cols<<endl;
  out_file.write("nrows ", 6);
  out_file.write(util_itochar((int)(cols)), strlen(util_itochar((int)cols)));
  out_file.write(" \n", 2);
  
  cout<<"xllcorner   "<<minx<<endl;
  out_file.write("xllcorner ", 10);
  out_file.write(util_ftochar(minx), strlen(util_ftochar(minx)));
  out_file.write(" \n", 2);
  
  cout<<"yllcorner  "<<miny<<endl;
  out_file.write("yllcorner ", 10);
  out_file.write(util_ftochar(miny), strlen(util_ftochar(miny)));
  out_file.write(" \n", 2);
  
  cout<<"cellsize  "<<cellSize<<endl;
  out_file.write("cellsize ", 9);
  out_file.write(util_ftochar(cellSize), strlen(util_ftochar(cellSize)) );
  out_file.write(" \n", 2);
 
  cout<<"NODATA_value 9 "<<endl;
  out_file.write("NODATA_value 9", 14);
 	out_file.write(" \n", 2);
  
  //// body 
  // read out the lines from the matrix backwards as that is how the esri grid
  // is supposed to be
  for(int i=1;i<=matrix.size();i++)
  {
    //cout<< (matrix.size())-i <<endl;
    cout<<matrix[(matrix.size())-i]<<endl;
    out_file.write( (matrix[(matrix.size())-i]).c_str(), strlen((matrix[(matrix.size())-i]).c_str()) );
    out_file.write(" \n", 2);
  }
  
  out_file.close();
  
}

/*****************************************************************************
 * function that returns the minumum x in an ns2grid
 * **************************************************************************/
float getNS2GridDX(vector<string> ns2grid)
{
   float dx;
   //start at one and look backwards for the differences
   for (int i=1; i<3; i++)
   {
  	char const * thisLine;
  	char const * lastLine;
    thisLine = ns2grid[i].c_str();
    lastLine = ns2grid[i-1].c_str();
    
    char this_x[20];
    char last_x[20];
		sscanf(thisLine,  "%s",  this_x);
		sscanf(lastLine,  "%s",  last_x);
    
    dx = (atof(this_x) - atof(last_x));
    
   }
   return(dx);

}

/*****************************************************************************
 * function that returns the minumum x in an ns2grid
 * **************************************************************************/
float getNS2GridXMin(vector<string> ns2grid)
{
   vector<string> v;
   for (int i=0; i<ns2grid.size(); i++)
   {
		char const * sLine;
    sLine = ns2grid[i].c_str();
    char  x[20];
    char  y[20];
    char  z[20];
    char  row[20];
    
		sscanf(sLine,  "%s %s %s %s",  x, y, z, row);
    //cout<<x<<endl;
    v.push_back(x);
    
   }
   float min = util_getVectorMin(v);
   return min;
        
}

/*****************************************************************************
 *  * function that returns the minumum y in an ns2grid
 *   **************************************************************************/
float getNS2GridYMin(vector<string> ns2grid)
{
   vector<string> v;
   for (int i=0; i<ns2grid.size(); i++)
   {
		char const * sLine;
    sLine = ns2grid[i].c_str();
    char  x[20];
    char  y[20];
    char  z[20];
    char  row[20];
    
		sscanf(sLine,  "%s %s %s %s",  x, y, z, row);
    //cout<<x<<endl;
    v.push_back(y);
    
   }
   float min = util_getVectorMin(v);
   return min;

}


/*****************************************************************************
 * function that returns the number of x-rows in an ns2grid
 * **************************************************************************/
float getNS2GridXRows(vector<string> ns2grid)
{
   vector<string> v;
   for (int i=0; i<ns2grid.size(); i++)
   {
		char const * sLine;
    sLine = ns2grid[i].c_str();
    char  x[20];
    char  y[20];
    char  z[20];
    char  row[20];
    
		sscanf(sLine,  "%s %s %s %s",  x, y, z, row);
    //cout<<x<<endl;
    v.push_back(row);
    
   }
   float max = util_getVectorMax(v);
   return max;
}

/*****************************************************************************
 * function that returns the number of y-cols in an ns2grid
 * **************************************************************************/
float getNS2GridYCols(vector<string> ns2grid)
{
  
   vector<string> v;
   for (int i=0; i<ns2grid.size(); i++)
   {
		char const * sLine;
    sLine = ns2grid[i].c_str();
    char  x[20];
    char  y[20];
    char  z[20];
    char  row[20];
    char  col[20];
    
		sscanf(sLine,  "%s %s %s %s %s",  x, y, z, row, col);
    //cout<<x<<endl;
    v.push_back(col);
    
   }
   float max = util_getVectorMax(v);
   return max;
}



/*****************************************************************************
 * function that returns the maximum numeric value from a string 
 * vector containing an array of floats
 * **************************************************************************/
float util_getVectorMax(vector<string> invec)
{
   float maxValue;
   float thisValue;
   //cout<< thisValue << " "<< maxValue <<endl;
        
   for (int i=0; i<invec.size(); i++)
   {
		char const * sLine;
		sLine = invec[i].c_str();
		char x[20];
		sscanf (sLine,  "%s",  x);
		thisValue =atof(x);
		//cout<< thisValue << " "<< maxValue <<endl;
    if (thisValue > maxValue || i == 0)
    {
            maxValue = thisValue;
    }
   } 
   return maxValue;
}

/*****************************************************************************
 * function that returns the minumum numeric value from a string 
 * vector containing an array of floats
 * 
 * **************************************************************************/
float util_getVectorMin(vector<string> invec)
{
   float minValue;
   float thisValue;
        
   for (int i=0; i<invec.size(); i++)
   {
		char const * sLine;
		sLine = invec[i].c_str();
		char x[20];
		sscanf (sLine,  "%s",  x);
		thisValue =atof(x);
		//cout<< thisValue << " "<< minValue <<endl;
    if (thisValue < minValue || i == 0)
    {
		   //cout<< i << ": "<<thisValue << " xx "<< minValue <<endl;
       minValue = thisValue;
    }
   } 
   return minValue;
}



/******************************************************************************
 * function that gets the bounding box for a data set
 * ****************************************************************************/

/******************************************************************************
 * getNS2DGrid -- function to return a vector of tokenizable strings 
 * containing x, y, z, nx, ny separated by white space 
 *
 * Parameters 
 * xmin
 * xmax
 * ymin
 * ymax
 * dx
 * dy
 * p
 * ***************************************************************************/
std::vector<string> getNS2DGrid( float xmin, float xmax, float ymin, float ymax, float dx, float dy, float p)
{
  int nx = (xmax-xmin)/dx; 
  int ny = (ymax-ymin)/dy;
  vector <string> out;
  
  for (int i = 0; i < ny+1; i++)
  {
	float y = ymin + (dy*i);
		for (int ii = 0; ii < nx+1; ii++)
		{
			float x = xmin + (dx*ii);
			//char * buf  = util_ftochar(x) ;
      char* line  = (char*)malloc( 4*40000 ); //fix this!!!
			//cout<<"clean: "<<line<<endl;
      strcpy(line, util_ftochar(x) );
      strcat(line, " " );
      strcat(line, util_ftochar(y) );
      strcat(line, " " );
      //strcat(line, util_itochar(rand()/1000000000) );
      strcat(line, util_itochar(p) );
      strcat(line, " " );
      strcat(line, util_itochar(ii+1) );
      strcat(line, " " );
      strcat(line, util_itochar(i+1) );
      //cout<<"IN: " << line <<endl;
	    out.push_back(line);
      free(line);
			line = NULL;
      //cout<<util_ftochar(x)<<" "<<y<<" "<<p<<" "<<ii<<" "<<i<<endl;
	  }
  }
  return(out);
}

/*****************************************************************************
 * function convert integer to char
 * 
 * ***************************************************************************/
char * util_itochar(int val)
{
  char * retStr;
  char buf[255];
  sprintf(buf, "%i", val);
  retStr = buf;
  return retStr;
}

/******************************************************************************
 * function that converts a float into a char pointer
 * 
 ****************************************************************************/
char * util_ftochar(float val)
{
  char * retStr;
  char buf[255];
  sprintf(buf, "%f", val);
  retStr = (char*)buf;
  return retStr;
}

