/**
 *  '$RCSfile$'
 *  '$Author$'
 *  '$Date$'
 *  '$Revision$'
 *
 *  For Details:
 *  http://kepler.ecoinformatics.org Copyright (c) 2004 The Regents of
 *  the University of California. All rights reserved. Permission is
 *  hereby granted, without written agreement and without license or
 *  royalty fees, to use, copy, modify, and distribute this software
 *  and its documentation for any purpose, provided that the above
 *  copyright notice and the following two paragraphs appear in all
 *  copies of this software. IN NO EVENT SHALL THE UNIVERSITY OF
 *  CALIFORNIA BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 *  INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
 *  THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 *  CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE
 *  UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 *  PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 *  CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 *  UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.kepler.gui;

import java.net.URL;
import javax.swing.SwingUtilities;

import org.ecoinformatics.util.Config;
import ptolemy.vergil.VergilApplication;

/** Main entry point for Kepler.
 *  This class reads the configuration ($KEPLER/lib/configuration.xml)
 *  for the path name to the splash screen and then brings up
 *  the splash screen.
 *  @author Kevin Ruland, Christopher Brooks
 *  @see SplashWindow
 */
public class KeplerApplication {

    /** Create a new instance of this application, passing it the
     *  command-line arguments.
     *  @param args The command-line arguments, see
     *  {@link ptolemy.vergil.VergilApplication}.  Usually, for Kepler,
     *  the first argument is "-kepler".
     */
    public static void main(final String[] args) {
        try {

            try {
                // FIXME: Config requires org.ecoinformatics.util
                // which introduces a problematic package dependency.
                String splashname = Config.getValue("//splash/image");
                final URL splashURL = ClassLoader.getSystemClassLoader()
                    .getResource(splashname);
                SwingUtilities.invokeAndWait(new Runnable() {
                        public void run() {
                            SplashWindow.splash(splashURL);
                        }
                    });
            } catch (Exception ex) {
                System.err.println("Failed to find splash screen image."
                        + "Ignoring, use the Java coffee cup");
                ex.printStackTrace();
            }            
            SplashWindow.invokeMain("ptolemy.vergil.VergilApplication", args);
            SplashWindow.disposeSplash();
        } catch (Throwable throwable) {
            System.err.println("Unable to start application.");
            throwable.printStackTrace();
        } finally {
            // Not sure if this is necessary.
            SplashWindow.disposeSplash();
        }
    }
}
