/* JGridletRemoteCreater 
 
  Copyright (c) 2005 The Regents of the University of California.
 All rights reserved.

  Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY


 */

package org.monash.griddles;

import java.io.File;
import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.BufferedInputStream;

import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.dom.DOMSource; 
import javax.xml.transform.stream.StreamResult; 
import java.io.FileOutputStream;


import org.kepler.objectmanager.library.LibraryIndex;
import org.kepler.objectmanager.library.LibraryIndexItem;
import org.kepler.objectmanager.library.LibraryIndexComponentItem;
import org.kepler.objectmanager.lsid.KeplerLSID;
import org.kepler.objectmanager.ActorMetadata;
import org.kepler.objectmanager.cache.ActorCacheObject;
import org.kepler.objectmanager.cache.CacheManager;
import org.kepler.objectmanager.cache.CacheException;
import org.kepler.objectmanager.cache.KARCacheObject;

import com.ibm.lsid.LSIDException;


 
public class GriddlesUtil
  {  
  /**
   * process the new ksw file into the actor library
   * @param kswFile the file to process
   */
  public void processKSW(File kswFile)
    {
    try
      {
      KeplerLSID[] actorLSIDs;
      CacheManager cache = CacheManager.getInstance();
      KARCacheObject kco = new KARCacheObject(kswFile);    
      LibraryIndex lindex; 
      actorLSIDs = kco.getActors();
      if(actorLSIDs != null)
        {                
        System.out.println("processing kar file: " + kswFile.getAbsolutePath());
        lindex = LibraryIndex.getInstance();        
        for(int j=0; j<actorLSIDs.length; j++)
          {             
          ActorCacheObject aco = (ActorCacheObject)cache.getObject(actorLSIDs[j]);                    
          ActorMetadata am = aco.getMetadata();          
          //build an entity for each actor in the ksw and add it to the actorLibrary
          LibraryIndexItem subtree = lindex.findItem(new KeplerLSID("urn:lsid:localhost:onto:1:1#PreConfiguredGridletActor"));
          LibraryIndexItem oldlsid = lindex.findItem(am.getName());            
          if(oldlsid != null)
            lindex.removeComponent(oldlsid.getLSID(), subtree);
          lindex.add(new LibraryIndexComponentItem(am.getName(), am.getLSID()), subtree);            
          //System.out.println("actor " + " loaded into the library in category " + subtree.getName() + " with id " +am.getId());
          }
        lindex.refresh();
        }
      }
    catch(CacheException ce)
      {
      System.out.println("Error with the cache: " + ce.getMessage());
      return;
      }
    catch(Exception e)
      {
      e.printStackTrace();
       System.out.println("Error creating ComponentEntity from actor " + "metadata: " + e.getMessage());
      return;
      }
    }
  public boolean executeCommand(String command, String file_prefix) 
    {
    Process process = null;
    PrintWriter pwr_err=null;
    PrintWriter pwr_out=null;
    try 
      {
      System.out.println("Executing external command " + command);
      process = Runtime.getRuntime().exec(command);
      if(file_prefix != null)
        {
        pwr_err = new PrintWriter(new BufferedWriter(new FileWriter(new File(file_prefix+"_err"), false)));        
        pwr_out = new PrintWriter(new BufferedWriter(new FileWriter(new File(file_prefix+"_out"), false)));
        }      
      }
    catch (IOException e) {
      System.out.println(e.toString());
      return false;
      }
    InputStream inputStream = process.getInputStream();
    BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
    InputStream errorStream = process.getErrorStream();
    BufferedInputStream bufferedErrorStream = new BufferedInputStream(errorStream);
    boolean ok = false;
    int exitValue = -999;

    while (!ok) 
      {
      try 
        {
        while (bufferedInputStream.available() > 0 || bufferedErrorStream.available() > 0) 
          {
          while (bufferedInputStream.available() > 0) 
            {
            if(file_prefix != null)
              pwr_out.print((char)bufferedInputStream.read());
            else  
              System.out.print((char)bufferedInputStream.read());
            }
          while (bufferedErrorStream.available() > 0) 
            {
            if(file_prefix != null)
              pwr_err.print((char)bufferedErrorStream.read());
            else
              System.out.print((char)bufferedErrorStream.read());
            }
          }
        }
      catch (IOException e) 
        {
        System.out.println("Couldn't read response");
        }
      try 
        {
        exitValue = process.exitValue();
        System.out.println("Process exit value= " + exitValue); 
        ok = true;
        }
      catch (IllegalThreadStateException e) 
        {
        }
      }
    if(ok)
      {
      // finished running
      if(exitValue == 0) 
        {
        System.out.println("Terminated without any errors");
        }
      else 
        {
        System.out.println("Exit code " + exitValue + " while performing command " + command);
        }
      }
    else 
      {
      process.destroy();
      }
    try 
      {
      while (bufferedInputStream.available() > 0) 
        {
        if(file_prefix != null)        
          pwr_out.print((char)bufferedInputStream.read());
        else
          System.out.print((char)bufferedInputStream.read());
        }

      while (bufferedErrorStream.available() > 0) 
        {
        if(file_prefix != null)        
          pwr_err.print((char)bufferedErrorStream.read());          
        else  
          System.out.print((char)bufferedErrorStream.read());
        }
      }
    catch (IOException e) 
      {
      System.out.println("Couldn't read response");
      }
    if(file_prefix != null)
      {
      pwr_out.flush();  
      pwr_out.close();
      pwr_err.flush();  
      pwr_err.close();      
      }      
    return exitValue == 0;
    }

  public boolean createXMLFile(String file_prefix, String exHost, String provider, String command, String arguments)
    {
    try
      {
         String stdout1 = file_prefix+"_out";
         String stderr1 = file_prefix +"_err";
         String xmlScript= file_prefix+"_xml.xml";
         
      //Create instance of DocumentBuilderFactory
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      //Get the DocumentBuilder
      DocumentBuilder docBuilder = factory.newDocumentBuilder();
      //Create blank DOM Document
      Document doc = docBuilder.newDocument();

      //create the root element
      Element root = doc.createElement("project");
      //all it to the xml tree
      doc.appendChild(root);    
      //create a comment
      //Comment comment = doc.createComment("This is comment");
      //add in the root element
      //root.appendChild(comment);

      //create child element
      Element childElement = doc.createElement("include");
      //Add the atribute to the child
      childElement.setAttribute("file","cogkit.xml");
      root.appendChild(childElement);
      childElement = doc.createElement("task:execute");
      childElement.setAttribute("executable", command);
      childElement.setAttribute("stdout", stdout1);
      childElement.setAttribute("stderr", stderr1);
      childElement.setAttribute("arguments", arguments);
      childElement.setAttribute("host", exHost);
      childElement.setAttribute("provider", provider);
      root.appendChild(childElement);
      childElement = doc.createElement("task:transfer");
      childElement.setAttribute("srchost", exHost);
      childElement.setAttribute("srcfile", stdout1);
      childElement.setAttribute("destfile", stdout1);
      childElement.setAttribute("desthost", "localhost");
      childElement.setAttribute("provider", "gridftp");
      root.appendChild(childElement);
    
      childElement = doc.createElement("task:transfer");
      childElement.setAttribute("srchost", exHost);
      childElement.setAttribute("srcfile", stderr1);
      childElement.setAttribute("destfile", stderr1);
      childElement.setAttribute("desthost", "localhost");
      childElement.setAttribute("provider", "gridftp");
      root.appendChild(childElement);

      TransformerFactory tranFactory = TransformerFactory.newInstance(); 
      Transformer aTransformer = tranFactory.newTransformer(); 

      Source src = new DOMSource(doc);
      FileOutputStream fos = new FileOutputStream(xmlScript); 
      Result dest = new StreamResult(fos); 
      aTransformer.transform(src, dest); 
      return true;
      }
    catch(Exception e)
      {
      System.out.println(e.getMessage());
      return false;
      }        
    }          
    
   public static String classPath;

  public void putJarFileInClassPath(File file)
    {
    if(file.isFile())
      {
      String name = file.getAbsolutePath();
      if(name.endsWith(".jar"))
        {
        classPath = classPath+file.pathSeparator+name;      
        //System.out.println(name);
        }
      }
    }   
  public String visitAllDirsAndFiles(File dir) 
    {
    putJarFileInClassPath(dir);
    if (dir.isDirectory()) 
      {
      String[] children = dir.list();
      for (int i=0; i<children.length; i++) 
        {
        visitAllDirsAndFiles(new File(dir, children[i]));
        }
      }
    return classPath;  
    }    
  }